/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   Use this program to experiment with two-dimensional rotations.

   Try to rotate a model around an arbitrary point inside of the model.

   Suppose you want to rotate a 2D model around the point (a,b) in the
   model's coordinates, and you want this rotation to happen at the
   point (c,d) in camera coordinates. You would do that with the following
   three transformations.

      model.modelTranslate(c, d, 0);
      model.modelRotate(theta, 0,0, 1);
      model.modelTranslate(-a, -b, 0);

   Try to do the following. The bronto's nose is approximately at the
   point (x, y)=(0.0834, 0.8540) in the bronto's model coordinates.
   Rotate the bronto around his nose but have his nose located at
   the point (0, 1.5) in camera coordinates.
*/
public class RotationTester
{
   public static void main(String[] args)
   {
      // Create a 2D Model object.
      Model m = new GRSModel(new File("assets/grs/bronto.grs"));

      // Set the model's color.
      m.setColor(Color.black);

      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the model to the Scene.
      scene.modelList.add(m);

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-1, +1, -1, +1, 24, 24);
      axes.setColor( Color.red );
      scene.modelList.add(axes);


      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);


      // Use orthographic projection
      scene.camera.perspective = false;

      for (int i = 0; i <= 36; i++)
      {
         // Rotate the center of the bronto around the point (-1, 2).
         m.model2Identity();
         m.modelScale(0.33, 0.33, 0.33); // scale the model to fit in the view rectangle
         // now rotate
         m.modelTranslate(-1, 2, 0);
         m.modelRotate(10*i, 0, 0, 1);
         m.modelTranslate(-0.5, -0.5, 0);

         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_RotationTester_a_Frame%02d.ppm", i));
      }

      for (int i = 0; i <= 36; i++)
      {
         // Rotate the bronto's nose around the point (0, 1.5).
         m.model2Identity();
         m.modelScale(0.33, 0.33, 0.33); // scale the model to fit in the view rectangle
         // now rotate
         m.modelTranslate(0, 1.5, 0);
         m.modelRotate(10*i, 0, 0, 1);
         m.modelTranslate(-0.0834, -0.8540, 0);

         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_RotationTester_b_Frame%02d.ppm", i));
      }
   }
}