/*
   Rotate a square around its corner at the origin.
*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   Rotate a square around its corner at the origin.

            y                             y

            |                             |
            |                            /|\
          1 +---------+                / 1+  \
            |         |              /    |    \
            |         |             <     |     >
            |         |              \    |    /
            |         |                \  |  /
            |         |                  \|/
    --------+---------+---> x     --------+--------+----> x
            |        1                    |        1
            |                             |
            |                             |
            |                             |


   In the code below, the main loop for creating the animation looks like this.

      for (int i = 0; i <= 60; i++)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));

         square.modelRotate(6, 0,0,1); // accumulate rotations in the model matrix
      }

   Each frame of the animation rotates the square another 6 degrees from the
   previous frame.

   Notice that this code "accumulates" the rotations in the square's model matrix.
   Here is what we mean by that.

   Let M denote the model matrix for the square. Initially, M is the identity matrix.

               M = [ 1  0  0  0 ]
                   [ 0  1  0  0 ]
                   [ 0  0  1  0 ]
                   [ 0  0  0  1 ]

   Let R(a) denote the 4x4 homogeneous matrix that rotates the xy-plane
   around the z-axis by angle a degrees counter-clockwise.

            R(a) = [ cos(a * (pi/180))   -sin(a * (pi/180))   0   0]
                   [ sin(a * (pi/180))    cos(a * (pi/180))   0   0]
                   [       0                    0             1   0]
                   [       0                    0             0   1]



   Each time we execute the line of code

       square.modelRotate(6, 0,0,1);

   we multiply the model matrix M on the RIGHT by the matrix R(6) and replace
   M by the result. Essentially, this line of code would look like this.

          M = M * R(6).

   Before the loop begins,
          M = I.
   After one iteration of the loop M is
          M = I * R(6).
   After two iterations, M is
          M = I * R(6) * R(6)
   but this simplifies to
          M = R(12).
   After i iterations, M simplifies to
          M = R(6*i).
   So the model matrix is just accumulating the total rotation.

   This is a very simple example and in the next example we will
   see that almost any other kind of rotation will not be this easy.
*/
public class Example_01
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-2, +2, -2, +2, 8, 8);
      axes.setColor( Color.red );
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -2);
      scene.addModel(axes);


      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);


      Model square = new Model();
      square.addLineSegment(new LineSegment(new Vertex(0, 0, 0), new Vertex(1, 0, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 0, 0), new Vertex(1, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 1, 0), new Vertex(0, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(0, 1, 0), new Vertex(0, 0, 0)));
      // Give the model a random color.
      square.setColorRandom();
      // Add the square to the Scene.
      scene.addModel( square );

      // Push the square away from where the camera is.
      square.modelTranslate(0, 0, -2);

      // Rotate the square around its corner at the origin..
      for (int i = 0; i <= 60; i++)
      {
         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));

         square.modelRotate(6, 0,0,1); // accumulate rotations in the model matrix
      }
   }
}
