/*
   Rotate a square around its center, with its center at the origin.
*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   Rotate a square around its center, with its center at the origin.

   In the last example we had a model of the unit square and we wanted
   to create an animation of rotating the square counter-clockwise
   around its corner at the origin.

           y                             y

           |                             |
           |                            /|\
         1 +---------+                / 1+  \
           |         |              /    |    \
           |         |             <     |     >
           |         |              \    |    /
           |         |                \  |  /
           |         |                  \|/
   --------+---------+---> x     --------+--------+----> x
           |        1                    |        1
           |                             |
           |                             |
           |                             |

   In this example, we want to create an animation of the unit square model
   rotating around its center point but with its center point located at the
   origin. So we need to take the square model and translate it down 1/2 unit
   and to the left 1/2 unit and then rotate counter-clockwise.

           y                             y                             y

           |                             |                             |
           |                             |                             |
         1 +---------+                 1 +                           1 +
           |         |                   |                             |
           |         |                   |                             |
           |    +    |              +----|----+                       /| \
           |         |              |    |    |                     /  |   \
           |         |              |    |    |                   /    |    \
   --------+---------+---> x     ---+----+----+----+---> x     --<-----+----->---+---> x
           |         1              |    |    |    1              \    |    /    1
           |                        |    |    |                     \  |  /
           |                        +----+----+                       \|/
           |                             |                             |

   Let us analyze how we would use matrix transformations to implement
   this animation.

   Let M denote the model matrix for the square. Initially, M is the
   identity matrix.

               M = [ 1  0  0  0 ]
                   [ 0  1  0  0 ]
                   [ 0  0  1  0 ]
                   [ 0  0  0  1 ]

   Let R(a) denote the 4x4 homogeneous matrix that rotates the xy-plane
   around the z-axis by angle a degrees counter-clockwise.

            R(a) = [ cos(a * (pi/180))   -sin(a * (pi/180))   0   0]
                   [ sin(a * (pi/180))    cos(a * (pi/180))   0   0]
                   [       0                    0             1   0]
                   [       0                    0             0   1]

   Let T(a,b,c) denote the 4x4 homogeneous matrix that translate by the
   amount a in the x-direction, the amount b in the y-direction, and the
   amount c in the z-direction.

        T(a,b,c) = [ 1  0  0  a ]
                   [ 0  1  0  b ]
                   [ 0  0  1  c ]
                   [ 0  0  0  1 ]

   To create the first frame of the animation, we want the square's model
   matrix to translate the square down and to the left.

       M = T(-0.5, -0.5, 0)

   To create the second frame of the animation, we want the square's model
   matrix to translate the square down and to the left and then rotate by
   6 degrees.

       M = R(6) * T(-0.5, -0.5, 0)

   Notice how the rotation matrix is on the left of the translation matrix. We
   want each vertex of the square model to be first translated and then rotated.
   That is, if v is a vertex from the square model we want

      M * v = R(6) * (T(-0.5, -0.5, 0) * v)

   For the third frame of the animation, we want the square's model matrix to
   first translate and then rotate by 12 degree.

       M = R(6) * R(6) * T(-0.5, -0.5, 0)
         = R(12) * T(-0.5, -0.5, 0)

   Notice again that the rotation matrix is multiplied on the left. For the
   i'th frame of the animation we want the square's model matrix to first
   translate and then rotate by i*6 degrees.

       M = R(6*i) * T(-0.5, -0.5, 0)

   Notice how the rotations are accumulating on the left of the square's model
   matrix. This might make you think that the loop for this animation would look
   like this.

      for (int i = 0; i <= 60; i++)
      {
         square.model2Identity();
         square.modelTranslate(-0.5, -0.5, 0);
         square.modelRotate(6*i, 0,0,1);

         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));
      }

   This code seems to say, "set the model matrix to the identity matrix, then
   translate, then rotate". But let us read the code very carefully. The first
   line in the body of the loop.

      square.model2Identity();

   means

      M = I

   Recall that the second line of code

      square.modelTranslate(-0.5, -0.5, 0);

   means the following formula (with the translation matrix multiplied on the right).

      M = M * T(-0.5, -0.5, 0)
        = I * T(-0.5, -0.5, 0)
        = T(-0.5, -0.5, 0)

   The next line of code,

      square.modelRotate(6*i, 0,0,1);

   then means (with the rotation matrix multiplied on the right)

      M = M * R(6*i)
        = T(-0.5, -0.5, 0) * R(6*i)

   But this is the opposite of what we want. Above we said that the model matrix
   should translate then rotate, which has the formula

      M = R(6*i) * T(-0.5, -0.5, 0)

   Here is what the loop for creating the animation should look like.

      for (int i = 0; i <= 60; i++)
      {
         square.model2Identity();
         square.modelRotate(6*i, 0,0,1);
         square.modelTranslate(-0.5, -0.5, 0);

         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));
      }

   If you read the code "backwards", from the bottom up, it says "translate and
   then rotate", which is what we want. But if you read it forwards, from the top
   down, it seems to say "rotate then translate". There is a way to make sense of
   both of these ways of reading the code and the second way, from the top down,
   is actually the preferred way to read the code once you understand how to
   interpret it.

   We know that the code

      square.model2Identity();
      square.modelRotate(6*i, 0,0,1);
      square.modelTranslate(-0.5, -0.5, 0);

   creates the following model matrix because of the rule about multiplying
   matrices on the right.

      M = I * R(6*i) * T(-0.5, -0.5, 0)

   Notice how, if you read the code from the bottom up, you are reading the formula
   from right-to-left. But if you read the code from the top down, you are reading
   the formula from left-to-right. Let v be a vertex from the square model. When we
   apply the model matrix to the vertex, we get this formula,

      M * v = I * R(6*i) * T(-0.5, -0.5, 0) * v
            = R(6*i) * ( T(-0.5, -0.5, 0) * v )

   which says that the vertex is first multiplied by the translation, then the result
   of the translation is multiplied by the rotation. In other words, "translate then rotate".
   So the "bottom up" (or right-to-left) way of reading the code tells us what the code does
   to each vertex.

   Here is how to read the code from the top down (or, how to read the formula from
   left-to-right). When read from the top down, we interpret each transformation as
   transforming the coordinate system. So
      square.modelRotate(6*i, 0,0,1);
   means rotate the coordinate system. And
      square.modelTranslate(-0.5, -0.5, 0);
   means, translate the rotated coordinate system.

   The next example program tries to visualize this transformation of the coordinate system.
*/
public class Example_02a
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-2, +2, -2, +2, 8, 8);
      axes.setColor( Color.red );
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -2);
      scene.addModel(axes);


      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);


      Model square = new Model();
      square.addLineSegment(new LineSegment(new Vertex(0, 0, 0), new Vertex(1, 0, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 0, 0), new Vertex(1, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 1, 0), new Vertex(0, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(0, 1, 0), new Vertex(0, 0, 0)));
      // Give the model a random color.
      square.setColorRandom();
      // Add the square to the Scene.
      scene.addModel( square );


      // Rotate the square around its center, with its center at the origin.
      for (int i = 0; i <= 60; i++)
      {
         square.model2Identity();
         // Push the square away from where the camera is.
         square.modelTranslate(0, 0, -2);
         square.modelRotate(6*i, 0,0,1); // notice the difference here
         square.modelTranslate(-0.5, -0.5, 0);

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));
      }
   }
}
